@@ -30,6 +30,7 @@ import (
 	goregexp "regexp" //nolint:depguard // The Prometheus client library requires us to pass a regexp from this package.
 	"runtime"
 	"runtime/debug"
+	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -289,6 +290,12 @@ func (c *flagConfig) setFeatureListOptions(logger *slog.Logger) error {
 				// See proposal: https://github.com/prometheus/proposals/pull/48
 				c.web.NativeOTLPDeltaIngestion = true
 				logger.Info("Enabling native ingestion of delta OTLP metrics, storing the raw sample values without conversion. WARNING: Delta support is in an early stage of development. The ingestion and querying process is likely to change over time.")
+			case "type-and-unit-labels":
+				c.scrape.EnableTypeAndUnitLabels = true
+				logger.Info("Experimental type and unit labels enabled")
+			case "use-uncached-io":
+				c.tsdb.UseUncachedIO = true
+				logger.Info("Experimental Uncached IO is enabled.")
 			default:
 				logger.Warn("Unknown option for --enable-feature", "option", o)
 			}
@@ -553,7 +560,7 @@ func main() {
 	a.Flag("scrape.discovery-reload-interval", "Interval used by scrape manager to throttle target groups updates.").
 		Hidden().Default("5s").SetValue(&cfg.scrape.DiscoveryReloadInterval)
 
-	a.Flag("enable-feature", "Comma separated feature names to enable. Valid options: exemplar-storage, expand-external-labels, memory-snapshot-on-shutdown, promql-per-step-stats, promql-experimental-functions, extra-scrape-metrics, auto-gomaxprocs, native-histograms, created-timestamp-zero-ingestion, concurrent-rule-eval, delayed-compaction, old-ui, otlp-deltatocumulative, promql-duration-expr. See https://prometheus.io/docs/prometheus/latest/feature_flags/ for more details.").
+	a.Flag("enable-feature", "Comma separated feature names to enable. Valid options: exemplar-storage, expand-external-labels, memory-snapshot-on-shutdown, promql-per-step-stats, promql-experimental-functions, extra-scrape-metrics, auto-gomaxprocs, native-histograms, created-timestamp-zero-ingestion, concurrent-rule-eval, delayed-compaction, old-ui, otlp-deltatocumulative, promql-duration-expr, use-uncached-io. See https://prometheus.io/docs/prometheus/latest/feature_flags/ for more details.").
 		Default("").StringsVar(&cfg.featureList)
 
 	a.Flag("agent", "Run Prometheus in 'Agent mode'.").BoolVar(&agentMode)
@@ -641,6 +648,17 @@ func main() {
 		logger.Error(fmt.Sprintf("Error loading dynamic scrape config files from config (--config.file=%q)", cfg.configFile), "file", absPath, "err", err)
 		os.Exit(2)
 	}
+
+	// Parse rule files to verify they exist and contain valid rules.
+	if err := rules.ParseFiles(cfgFile.RuleFiles); err != nil {
+		absPath, pathErr := filepath.Abs(cfg.configFile)
+		if pathErr != nil {
+			absPath = cfg.configFile
+		}
+		logger.Error(fmt.Sprintf("Error loading rule file patterns from config (--config.file=%q)", cfg.configFile), "file", absPath, "err", err)
+		os.Exit(2)
+	}
+
 	if cfg.tsdb.EnableExemplarStorage {
 		if cfgFile.StorageConfig.ExemplarsConfig == nil {
 			cfgFile.StorageConfig.ExemplarsConfig = &config.DefaultExemplarsConfig
@@ -651,6 +669,32 @@ func main() {
 		cfg.tsdb.OutOfOrderTimeWindow = cfgFile.StorageConfig.TSDBConfig.OutOfOrderTimeWindow
 	}
 
+	// Set Go runtime parameters before we get too far into initialization.
+	updateGoGC(cfgFile, logger)
+	if cfg.maxprocsEnable {
+		l := func(format string, a ...interface{}) {
+			logger.Info(fmt.Sprintf(strings.TrimPrefix(format, "maxprocs: "), a...), "component", "automaxprocs")
+		}
+		if _, err := maxprocs.Set(maxprocs.Logger(l)); err != nil {
+			logger.Warn("Failed to set GOMAXPROCS automatically", "component", "automaxprocs", "err", err)
+		}
+	}
+
+	if cfg.memlimitEnable {
+		if _, err := memlimit.SetGoMemLimitWithOpts(
+			memlimit.WithRatio(cfg.memlimitRatio),
+			memlimit.WithProvider(
+				memlimit.ApplyFallback(
+					memlimit.FromCgroup,
+					memlimit.FromSystem,
+				),
+			),
+			memlimit.WithLogger(logger.With("component", "automemlimit")),
+		); err != nil {
+			logger.Warn("automemlimit", "msg", "Failed to set GOMEMLIMIT automatically", "err", err)
+		}
+	}
+
 	// Now that the validity of the config is established, set the config
 	// success metrics accordingly, although the config isn't really loaded
 	// yet. This will happen later (including setting these metrics again),
@@ -801,29 +845,6 @@ func main() {
 		ruleManager *rules.Manager
 	)
 
-	if cfg.maxprocsEnable {
-		l := func(format string, a ...interface{}) {
-			logger.Info(fmt.Sprintf(strings.TrimPrefix(format, "maxprocs: "), a...), "component", "automaxprocs")
-		}
-		if _, err := maxprocs.Set(maxprocs.Logger(l)); err != nil {
-			logger.Warn("Failed to set GOMAXPROCS automatically", "component", "automaxprocs", "err", err)
-		}
-	}
-
-	if cfg.memlimitEnable {
-		if _, err := memlimit.SetGoMemLimitWithOpts(
-			memlimit.WithRatio(cfg.memlimitRatio),
-			memlimit.WithProvider(
-				memlimit.ApplyFallback(
-					memlimit.FromCgroup,
-					memlimit.FromSystem,
-				),
-			),
-		); err != nil {
-			logger.Warn("automemlimit", "msg", "Failed to set GOMEMLIMIT automatically", "err", err)
-		}
-	}
-
 	if !agentMode {
 		opts := promql.EngineOpts{
 			Logger:                   logger.With("component", "query engine"),
@@ -839,6 +860,7 @@ func main() {
 			EnableNegativeOffset:     true,
 			EnablePerStepStats:       cfg.enablePerStepStats,
 			EnableDelayedNameRemoval: cfg.promqlEnableDelayedNameRemoval,
+			EnableTypeAndUnitLabels:  cfg.scrape.EnableTypeAndUnitLabels,
 		}
 
 		queryEngine = promql.NewEngine(opts)
@@ -1509,6 +1531,14 @@ func reloadConfig(filename string, enableExemplarStorage bool, logger *slog.Logg
 		return fmt.Errorf("one or more errors occurred while applying the new configuration (--config.file=%q)", filename)
 	}
 
+	updateGoGC(conf, logger)
+
+	noStepSuqueryInterval.Set(conf.GlobalConfig.EvaluationInterval)
+	timingsLogger.Info("Completed loading of configuration file", "filename", filename, "totalDuration", time.Since(start))
+	return nil
+}
+
+func updateGoGC(conf *config.Config, logger *slog.Logger) {
 	oldGoGC := debug.SetGCPercent(conf.Runtime.GoGC)
 	if oldGoGC != conf.Runtime.GoGC {
 		logger.Info("updated GOGC", "old", oldGoGC, "new", conf.Runtime.GoGC)
@@ -1519,10 +1549,6 @@ func reloadConfig(filename string, enableExemplarStorage bool, logger *slog.Logg
 	} else {
 		os.Setenv("GOGC", "off")
 	}
-
-	noStepSuqueryInterval.Set(conf.GlobalConfig.EvaluationInterval)
-	timingsLogger.Info("Completed loading of configuration file", "filename", filename, "totalDuration", time.Since(start))
-	return nil
 }
 
 func startsOrEndsWithQuote(s string) bool {
@@ -1833,6 +1859,7 @@ type tsdbOptions struct {
 	EnableDelayedCompaction        bool
 	CompactionDelayMaxPercent      int
 	EnableOverlappingCompaction    bool
+	UseUncachedIO                  bool
 }
 
 func (opts tsdbOptions) ToTSDBOptions() tsdb.Options {
@@ -1856,6 +1883,7 @@ func (opts tsdbOptions) ToTSDBOptions() tsdb.Options {
 		EnableDelayedCompaction:        opts.EnableDelayedCompaction,
 		CompactionDelayMaxPercent:      opts.CompactionDelayMaxPercent,
 		EnableOverlappingCompaction:    opts.EnableOverlappingCompaction,
+		UseUncachedIO:                  opts.UseUncachedIO,
 	}
 }
 
@@ -1914,10 +1942,8 @@ func (p *rwProtoMsgFlagParser) Set(opt string) error {
 	if err := t.Validate(); err != nil {
 		return err
 	}
-	for _, prev := range *p.msgs {
-		if prev == t {
-			return fmt.Errorf("duplicated %v flag value, got %v already", t, *p.msgs)
-		}
+	if slices.Contains(*p.msgs, t) {
+		return fmt.Errorf("duplicated %v flag value, got %v already", t, *p.msgs)
 	}
 	*p.msgs = append(*p.msgs, t)
 	return nil
