@@ -510,7 +510,7 @@ func TestCheckRules(t *testing.T) {
 		os.Stdin = r
 
 		exitCode := CheckRules(newRulesLintConfig(lintOptionDuplicateRules, false, false))
-		require.Equal(t, successExitCode, exitCode, "")
+		require.Equal(t, successExitCode, exitCode)
 	})
 
 	t.Run("rules-bad", func(t *testing.T) {
@@ -532,7 +532,7 @@ func TestCheckRules(t *testing.T) {
 		os.Stdin = r
 
 		exitCode := CheckRules(newRulesLintConfig(lintOptionDuplicateRules, false, false))
-		require.Equal(t, failureExitCode, exitCode, "")
+		require.Equal(t, failureExitCode, exitCode)
 	})
 
 	t.Run("rules-lint-fatal", func(t *testing.T) {
@@ -554,7 +554,7 @@ func TestCheckRules(t *testing.T) {
 		os.Stdin = r
 
 		exitCode := CheckRules(newRulesLintConfig(lintOptionDuplicateRules, true, false))
-		require.Equal(t, lintErrExitCode, exitCode, "")
+		require.Equal(t, lintErrExitCode, exitCode)
 	})
 }
 
@@ -572,19 +572,19 @@ func TestCheckRulesWithRuleFiles(t *testing.T) {
 	t.Run("rules-good", func(t *testing.T) {
 		t.Parallel()
 		exitCode := CheckRules(newRulesLintConfig(lintOptionDuplicateRules, false, false), "./testdata/rules.yml")
-		require.Equal(t, successExitCode, exitCode, "")
+		require.Equal(t, successExitCode, exitCode)
 	})
 
 	t.Run("rules-bad", func(t *testing.T) {
 		t.Parallel()
 		exitCode := CheckRules(newRulesLintConfig(lintOptionDuplicateRules, false, false), "./testdata/rules-bad.yml")
-		require.Equal(t, failureExitCode, exitCode, "")
+		require.Equal(t, failureExitCode, exitCode)
 	})
 
 	t.Run("rules-lint-fatal", func(t *testing.T) {
 		t.Parallel()
 		exitCode := CheckRules(newRulesLintConfig(lintOptionDuplicateRules, true, false), "./testdata/prometheus-rules.lint.yml")
-		require.Equal(t, lintErrExitCode, exitCode, "")
+		require.Equal(t, lintErrExitCode, exitCode)
 	})
 }
 
