@@ -695,7 +695,7 @@ func TestTargetUpdatesOrder(t *testing.T) {
 			for x := 0; x < totalUpdatesCount; x++ {
 				select {
 				case <-ctx.Done():
-					require.FailNow(t, "%d: no update arrived within the timeout limit", x)
+					t.Fatalf("%d: no update arrived within the timeout limit", x)
 				case tgs := <-provUpdates:
 					discoveryManager.updateGroup(poolKey{setName: strconv.Itoa(i), provider: tc.title}, tgs)
 					for _, got := range discoveryManager.allGroups() {
@@ -769,12 +769,10 @@ func verifyPresence(t *testing.T, tSets map[poolKey]map[string]*targetgroup.Grou
 			}
 		}
 	}
-	if match != present {
-		msg := ""
-		if !present {
-			msg = "not"
-		}
-		require.FailNow(t, "%q should %s be present in Targets labels: %q", label, msg, mergedTargets)
+	if present {
+		require.Truef(t, match, "%q must be present in Targets labels: %q", label, mergedTargets)
+	} else {
+		require.Falsef(t, match, "%q must be absent in Targets labels: %q", label, mergedTargets)
 	}
 }
 
@@ -1091,9 +1089,9 @@ func TestTargetSetRecreatesEmptyStaticConfigs(t *testing.T) {
 	targetGroups, ok := discoveryManager.targets[p]
 	require.True(t, ok, "'%v' should be present in targets", p)
 	// Otherwise the targetGroups will leak, see https://github.com/prometheus/prometheus/issues/12436.
-	require.Empty(t, targetGroups, 0, "'%v' should no longer have any associated target groups", p)
+	require.Empty(t, targetGroups, "'%v' should no longer have any associated target groups", p)
 	require.Len(t, syncedTargets, 1, "an update with no targetGroups should still be sent.")
-	require.Empty(t, syncedTargets["prometheus"], 0)
+	require.Empty(t, syncedTargets["prometheus"])
 }
 
 func TestIdenticalConfigurationsAreCoalesced(t *testing.T) {
@@ -1373,10 +1371,10 @@ func TestCoordinationWithReceiver(t *testing.T) {
 				time.Sleep(expected.delay)
 				select {
 				case <-ctx.Done():
-					require.FailNow(t, "step %d: no update received in the expected timeframe", i)
+					t.Fatalf("step %d: no update received in the expected timeframe", i)
 				case tgs, ok := <-mgr.SyncCh():
 					require.True(t, ok, "step %d: discovery manager channel is closed", i)
-					require.Equal(t, len(expected.tgs), len(tgs), "step %d: targets mismatch", i)
+					require.Len(t, tgs, len(expected.tgs), "step %d: targets mismatch", i)
 
 					for k := range expected.tgs {
 						_, ok := tgs[k]
