@@ -146,6 +146,24 @@ func (node *BinaryExpr) getMatchingStr() string {
 	return matching
 }
 
+func (node *DurationExpr) String() string {
+	var expr string
+	if node.LHS == nil {
+		// This is a unary negative duration expression.
+		expr = fmt.Sprintf("%s%s", node.Op, node.RHS)
+	} else {
+		expr = fmt.Sprintf("%s %s %s", node.LHS, node.Op, node.RHS)
+	}
+	if node.Wrapped {
+		return fmt.Sprintf("(%s)", expr)
+	}
+	return expr
+}
+
+func (node *DurationExpr) ShortString() string {
+	return node.Op.String()
+}
+
 func (node *Call) String() string {
 	return fmt.Sprintf("%s(%s)", node.Func.Name, node.Args)
 }
@@ -159,6 +177,8 @@ func (node *MatrixSelector) atOffset() (string, string) {
 	vecSelector := node.VectorSelector.(*VectorSelector)
 	offset := ""
 	switch {
+	case vecSelector.OriginalOffsetExpr != nil:
+		offset = fmt.Sprintf(" offset %s", vecSelector.OriginalOffsetExpr)
 	case vecSelector.OriginalOffset > time.Duration(0):
 		offset = fmt.Sprintf(" offset %s", model.Duration(vecSelector.OriginalOffset))
 	case vecSelector.OriginalOffset < time.Duration(0):
@@ -181,21 +201,30 @@ func (node *MatrixSelector) String() string {
 	// Copy the Vector selector before changing the offset
 	vecSelector := *node.VectorSelector.(*VectorSelector)
 	// Do not print the @ and offset twice.
-	offsetVal, atVal, preproc := vecSelector.OriginalOffset, vecSelector.Timestamp, vecSelector.StartOrEnd
+	offsetVal, offsetExprVal, atVal, preproc := vecSelector.OriginalOffset, vecSelector.OriginalOffsetExpr, vecSelector.Timestamp, vecSelector.StartOrEnd
 	vecSelector.OriginalOffset = 0
+	vecSelector.OriginalOffsetExpr = nil
 	vecSelector.Timestamp = nil
 	vecSelector.StartOrEnd = 0
 
-	str := fmt.Sprintf("%s[%s]%s%s", vecSelector.String(), model.Duration(node.Range), at, offset)
+	rangeStr := model.Duration(node.Range).String()
+	if node.RangeExpr != nil {
+		rangeStr = node.RangeExpr.String()
+	}
+	str := fmt.Sprintf("%s[%s]%s%s", vecSelector.String(), rangeStr, at, offset)
 
-	vecSelector.OriginalOffset, vecSelector.Timestamp, vecSelector.StartOrEnd = offsetVal, atVal, preproc
+	vecSelector.OriginalOffset, vecSelector.OriginalOffsetExpr, vecSelector.Timestamp, vecSelector.StartOrEnd = offsetVal, offsetExprVal, atVal, preproc
 
 	return str
 }
 
 func (node *MatrixSelector) ShortString() string {
 	at, offset := node.atOffset()
-	return fmt.Sprintf("[%s]%s%s", model.Duration(node.Range), at, offset)
+	rangeStr := model.Duration(node.Range).String()
+	if node.RangeExpr != nil {
+		rangeStr = node.RangeExpr.String()
+	}
+	return fmt.Sprintf("[%s]%s%s", rangeStr, at, offset)
 }
 
 func (node *SubqueryExpr) String() string {
@@ -211,9 +240,13 @@ func (node *SubqueryExpr) getSubqueryTimeSuffix() string {
 	step := ""
 	if node.Step != 0 {
 		step = model.Duration(node.Step).String()
+	} else if node.StepExpr != nil {
+		step = node.StepExpr.String()
 	}
 	offset := ""
 	switch {
+	case node.OriginalOffsetExpr != nil:
+		offset = fmt.Sprintf(" offset %s", node.OriginalOffsetExpr)
 	case node.OriginalOffset > time.Duration(0):
 		offset = fmt.Sprintf(" offset %s", model.Duration(node.OriginalOffset))
 	case node.OriginalOffset < time.Duration(0):
@@ -228,11 +261,21 @@ func (node *SubqueryExpr) getSubqueryTimeSuffix() string {
 	case node.StartOrEnd == END:
 		at = " @ end()"
 	}
-	return fmt.Sprintf("[%s:%s]%s%s", model.Duration(node.Range), step, at, offset)
+	rangeStr := model.Duration(node.Range).String()
+	if node.RangeExpr != nil {
+		rangeStr = node.RangeExpr.String()
+	}
+	return fmt.Sprintf("[%s:%s]%s%s", rangeStr, step, at, offset)
 }
 
 func (node *NumberLiteral) String() string {
-	return fmt.Sprint(node.Val)
+	if node.Duration {
+		if node.Val < 0 {
+			return fmt.Sprintf("-%s", model.Duration(-node.Val*1e9).String())
+		}
+		return model.Duration(node.Val * 1e9).String()
+	}
+	return strconv.FormatFloat(node.Val, 'f', -1, 64)
 }
 
 func (node *ParenExpr) String() string {
@@ -265,6 +308,8 @@ func (node *VectorSelector) String() string {
 	}
 	offset := ""
 	switch {
+	case node.OriginalOffsetExpr != nil:
+		offset = fmt.Sprintf(" offset %s", node.OriginalOffsetExpr)
 	case node.OriginalOffset > time.Duration(0):
 		offset = fmt.Sprintf(" offset %s", model.Duration(node.OriginalOffset))
 	case node.OriginalOffset < time.Duration(0):
