@@ -36,8 +36,7 @@ const defaultZeroThreshold = 1e-128
 // addExponentialHistogramDataPoints adds OTel exponential histogram data points to the corresponding time series
 // as native histogram samples.
 func (c *PrometheusConverter) addExponentialHistogramDataPoints(ctx context.Context, dataPoints pmetric.ExponentialHistogramDataPointSlice,
-	resource pcommon.Resource, settings Settings, promName string,
-	temporality pmetric.AggregationTemporality,
+	resource pcommon.Resource, settings Settings, promName string, temporality pmetric.AggregationTemporality,
 ) (annotations.Annotations, error) {
 	var annots annotations.Annotations
 	for x := 0; x < dataPoints.Len(); x++ {
@@ -252,8 +251,7 @@ func convertBucketsLayout(bucketCounts []uint64, offset, scaleDown int32, adjust
 }
 
 func (c *PrometheusConverter) addCustomBucketsHistogramDataPoints(ctx context.Context, dataPoints pmetric.HistogramDataPointSlice,
-	resource pcommon.Resource, settings Settings, promName string,
-	temporality pmetric.AggregationTemporality,
+	resource pcommon.Resource, settings Settings, promName string, temporality pmetric.AggregationTemporality,
 ) (annotations.Annotations, error) {
 	var annots annotations.Annotations
 
