@@ -421,10 +421,10 @@ func TestPersistence_index_e2e(t *testing.T) {
 	for k, v := range labelPairs {
 		sort.Strings(v)
 
-		res, err := ir.SortedLabelValues(ctx, k)
+		res, err := ir.SortedLabelValues(ctx, k, nil)
 		require.NoError(t, err)
 
-		require.Equal(t, len(v), len(res))
+		require.Len(t, res, len(v))
 		for i := 0; i < len(v); i++ {
 			require.Equal(t, v[i], res[i])
 		}
