@@ -347,6 +347,24 @@ export const functionIdentifierTerms = [
     info: 'Return the minimum value over time for input series',
     type: 'function',
   },
+  {
+    label: 'ts_of_max_over_time',
+    detail: 'function',
+    info: 'Return the timestamp of the maximum value over time for input series',
+    type: 'function',
+  },
+  {
+    label: 'ts_of_min_over_time',
+    detail: 'function',
+    info: 'Return the timestamp of the minimum value over time for input series',
+    type: 'function',
+  },
+  {
+    label: 'ts_of_last_over_time',
+    detail: 'function',
+    info: 'Return the timestamp of the last value over time for input series',
+    type: 'function',
+  },
   {
     label: 'minute',
     detail: 'function',
